/*
 * LXQt - a lightweight, Qt based, desktop toolset
 * Copyright (C) 2016  Luís Pereira <luis.artur.pereira@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301  USA
 */

#ifndef LXQTPANELAPPLICATION_P_H
#define LXQTPANELAPPLICATION_P_H

#include "lxqtpanelapplication.h"

namespace LXQt {
class Settings;
}

class ILXQtAbstractWMInterface;

class LXQtPanelApplicationPrivate {
    Q_DECLARE_PUBLIC(LXQtPanelApplication)
public:

    LXQtPanelApplicationPrivate(LXQtPanelApplication *q);
    ~LXQtPanelApplicationPrivate() {};

    LXQt::Settings *mSettings;
    ILXQtAbstractWMInterface *mWMBackend;

    ILXQtPanel::Position computeNewPanelPosition(const LXQtPanel *p, const int screenNum);

    void loadBackend();

private:
    LXQtPanelApplication *const q_ptr;
};

#endif // LXQTPANELAPPLICATION_P_H
